﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.IO;

namespace ComplexNumber
{
    public struct SizeD
    {
        private double mWidth;
        private double mHeight;

        public SizeD(double Width, double Height)
        {
            mWidth = Width;
            mHeight = Height;
        }

        public double Width
        {
            get { return mWidth; }
            set { mWidth = value; }
        }

        public double Height
        {
            get { return mHeight; }
            set { mHeight = value; }
        }
    }

    public struct PointD
    {
        private double mX;
        private double mY;

        public PointD(double X, double Y)
        {
            mX = X;
            mY = Y;
        }

        public double Y
        {
            get { return mY; }
            set { mY = value; }
        }

        public double X
        {
            get { return mX; }
            set { mX = value; }
        }
    }

    public struct RectangleD
    {
        private double mWidth;
        private double mHeight;
        private double mX;
        private double mY;

        public RectangleD(double Width, double Height, double X, double Y)
        {
            mWidth = Width;
            mHeight = Height;
            mX = X;
            mY = Y;
        }

        public double Y
        {
            get { return mY; }
            set { mY = value; }
        }

        public double X
        {
            get { return mX; }
            set { mX = value; }
        }

        public double Width
        {
            get { return mWidth; }
            set { mWidth = value; }
        }

        public double Height
        {
            get { return mHeight; }
            set { mHeight = value; }
        }
    }

    class classMandel
    {
        private Size mImageSize = new Size(640, 480);
        private RectangleD mPosition = new RectangleD(4, 3, -2.5, -1.5);
        private ComplexNumber mStartVal = new ComplexNumber(0,0);
        private ColorPalette mPal = new Bitmap(1, 1, PixelFormat.Format8bppIndexed).Palette;
        private double mProg = 0;
        private int mPower = 2;
        private int mIterations = 300;

        public ColorPalette Palette
        {
            get { return mPal; }
            set { mPal = value; }
        }

        public ComplexNumber StartPos
        {
            get { return mStartVal; }
            set { mStartVal = value; }
        }

        public int Power
        {
            get { return mPower; }
            set { mPower = value; }
        }

        public RectangleD ViewRect
        {
            get { return mPosition; }
            set { mPosition = value; }
        }

        public Size ImageSize
        {
            get { return mImageSize; }
            set { mImageSize = value; }
        }

        public int Iterations
        {
            get { return mIterations; }
            set { mIterations = value; }
        }

        public double Progress
        { get { return mProg; } }

        private ComplexNumber Recur(ComplexNumber z, ComplexNumber c)
        {//Hier ist nur die Rekursionsgleichung extra (für etwaige veränderungen)
            return (z ^ mPower) + c;
        }

        private int DoRecur(ComplexNumber Start/*Startwert für z*/, ComplexNumber c/*Zweitmember der Rekursionsgleichung*/)
        {//Diese Routine führt die Rekursion durch
            int n = 0;
            ComplexNumber z = Start;

            while (n < mIterations && !(z.ImaginaerPart * z.ImaginaerPart + z.RealPart * z.RealPart > 4)) //Wurzelziehen ist langsam
            {
                n++;
                z = Recur(z, c);
            }

            return n;
        }

        private SizeD GetStep()
        {//schrittgröße im Koordinatensystem errechnen, damit jedes Pixel richtig berechnet wird
            return new SizeD(mPosition.Width / mImageSize.Width, mPosition.Height / mImageSize.Height);
        }

        public ColorPalette GetPalFile(string FileName)
        {//Einlesen eines *.pal Files (Aufbau: 256*4 Bytes = 256 Farben)
            Bitmap bmp = new Bitmap(1, 1, PixelFormat.Format8bppIndexed);
            ColorPalette pal = bmp.Palette;//Palette mit 256 Einträgen übernehmen
            FileStream stream = new FileStream(FileName, FileMode.Open);

            for (int i = 0; i < pal.Entries.Length; i++)
            {
                pal.Entries[i] = Color.FromArgb(stream.ReadByte(), stream.ReadByte(), stream.ReadByte(), stream.ReadByte());
            }

            stream.Close();

            return pal;
        }

        public RectangleD GetNewRect(Point Down/*Punkt, an dem die maus geklickt wurde*/, Point Up/*Punkt, an dem die Maus losgelassen wurde*/)
        {//Berechnet den neuen Ausschnitt im Koordinatensystem für punkte auf der Zeichenfläche (Punkte in Pixel)
            SizeD Steps = GetStep();
            PointD DownP = new PointD(mPosition.X + (Steps.Width * Down.X), mPosition.Y + (Steps.Height * Down.Y));
            PointD UpP = new PointD(mPosition.X + (Steps.Width * Up.X), mPosition.Y + (Steps.Height * Up.Y));
            double minX = Math.Min(DownP.X, UpP.X);
            double minY = Math.Min(DownP.Y, UpP.Y);
            double maxX = Math.Max(DownP.X, UpP.X);
            double maxY = Math.Max(DownP.Y, UpP.Y);

            return new RectangleD(maxX - minX, maxY - minY, minX, minY);
        }

        public Bitmap DrawMandelbrot()
        {//Zeichnet die Mandelbrotmenge mitden voreingestellten Parametern
            Bitmap bmp = new Bitmap(mImageSize.Width, mImageSize.Height, PixelFormat.Format8bppIndexed);

            bmp.Palette = mPal;

            SizeD Steps = GetStep();
            double NColorP;
            int tmp;

            BitmapData data0 = bmp.LockBits(new Rectangle(Point.Empty, mImageSize), ImageLockMode.ReadWrite, PixelFormat.Format8bppIndexed);
            byte[] ColorData = new byte[data0.Stride * mImageSize.Height];
            Marshal.Copy(data0.Scan0, ColorData, 0, ColorData.Length);

            for (int x = 0; x < mImageSize.Width; x++)
            {//Jedes Pixel durchlaufen
                for (int y = 0; y < mImageSize.Height; y++)
                {
                    tmp = DoRecur(mStartVal,  new ComplexNumber(mPosition.X + (Steps.Width * x), mPosition.Y + (Steps.Height * y)));//Rekursion durchführen
                    NColorP = (tmp * 1D) / mIterations;

                    byte s = (byte)(NColorP * 255);

                    ColorData[x + y * data0.Stride] = s;//Einfärben 
                    //(TRICK: da 8Bit Indexed Bitmap können verschiedene Farbpaletten geladen werden)
                }

                mProg = (x * 1D) / mImageSize.Width;
                Application.DoEvents();
            }

            Marshal.Copy(ColorData, 0, data0.Scan0, ColorData.Length);
            bmp.UnlockBits(data0);

            return bmp;
        }
    }
}
