﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;

namespace ComplexNumber
{
    public partial class frmMain : Form
    {
        private classMandel mMandelClass = new classMandel();
        private Point mClickPoint;
        private Bitmap mMandel;
        private Thread mDrawThread;
        private Point mMandelPos = new Point(0, 0);

        public frmMain()
        {
            InitializeComponent();
        }

        private void ColorRotation()
        {
            ColorPalette pal = mMandel.Palette;
            Color fstTmp = pal.Entries[0];

            for (int i = 1; i < pal.Entries.Length-1; i++)
                pal.Entries[i - 1] = pal.Entries[i];

            pal.Entries[pal.Entries.Length - 2] = fstTmp;

            mMandel.Palette = pal;
        }

        #region init Drawing Thread
        private void DrawMandel()
        {
            mMandel = mMandelClass.DrawMandelbrot();
            picFrakt.Image = CenterMandel();
            tmrProgress.Stop();
        }

        private void SetMandel()
        {
            if (mDrawThread != null)
                if (mDrawThread.IsAlive)
                    mDrawThread.Abort();

            tmrProgress.Start();
            mDrawThread = new Thread(new ThreadStart(DrawMandel));
            mDrawThread.Start();
        }

        private void DrawTimer(object sender, EventArgs e)
        {
            prgProg.Value = (int)(mMandelClass.Progress * 100);
        }
        #endregion

        private Bitmap CenterMandel()
        {
            Bitmap ret = new Bitmap(Math.Max(1,picFrakt.Width), Math.Max(1,picFrakt.Height));

            if (mMandel != null)
            {
                mMandelPos = new Point(ret.Width / 2 - mMandel.Width / 2, ret.Height / 2 - mMandel.Height / 2);

                using (Graphics gr = Graphics.FromImage(ret))
                {
                    gr.DrawImageUnscaled(mMandel, mMandelPos);
                    gr.DrawRectangle(Pens.Gray, new Rectangle(mMandelPos, mMandel.Size));
                }
            }

            return ret;
        }

        private Bitmap CenterMandel(Rectangle SelectionRect)
        {
            Bitmap ret = new Bitmap(Math.Max(1, picFrakt.Width), Math.Max(1, picFrakt.Height));

            if (mMandel != null)
            {
                mMandelPos = new Point(ret.Width / 2 - mMandel.Width / 2, ret.Height / 2 - mMandel.Height / 2);

                SelectionRect.X += mMandelPos.X;
                SelectionRect.Y += mMandelPos.Y;

                using (Graphics gr = Graphics.FromImage(ret))
                {
                    gr.Clear(Color.Transparent);
                    gr.DrawImageUnscaled(mMandel, mMandelPos);
                    gr.DrawRectangle(Pens.Gray, new Rectangle(mMandelPos, mMandel.Size));

                    Pen rectPen = new Pen(Color.Black);
                    rectPen.DashStyle = DashStyle.Dash;

                    gr.DrawRectangle(Pens.White , SelectionRect);
                    gr.DrawRectangle(rectPen, SelectionRect);
                }
            }

            return ret;
        }


        private void button1_Click(object sender, EventArgs e)
        {
            SetMandel();
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            int x = Math.Max(Math.Min(e.Location.X - mMandelPos.X, mMandel.Width), 0);
            int y = Math.Max(Math.Min(e.Location.Y - mMandelPos.Y, mMandel.Height), 0);

            mClickPoint = new Point(x, y);
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            int x = Math.Max(Math.Min(e.Location.X - mMandelPos.X, mMandel.Width), 0);
            int y = Math.Max(Math.Min(e.Location.Y - mMandelPos.Y, mMandel.Height), 0);

            mMandelClass.ViewRect = mMandelClass.GetNewRect(mClickPoint, new Point(x, y));
            RefreshBoxes();
            SetMandel();
        }

        private void schließenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Environment.Exit(0);
        }

        private void bildSpeichernToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();

            dlg.Filter = "Bitmap|*.bmp|JPEG|*.jpg|PNG|*.png";

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                switch (dlg.FilterIndex)
                {
                    case 1:
                        mMandel.Save(dlg.FileName, ImageFormat.Bmp);
                        break;
                    case 2:
                        mMandel.Save(dlg.FileName, ImageFormat.Jpeg);
                        break;
                    case 3:
                        mMandel.Save(dlg.FileName, ImageFormat.Png);
                        break;
                }
            }
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (mDrawThread != null)
                if (mDrawThread.IsAlive)
                    if (MessageBox.Show("Es wird noch gerendert. Wollen sie wirklich schließen?", "Schließen", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.No)
                        e.Cancel = true;
                    else
                        mDrawThread.Abort();
        }

        private void picFrakt_SizeChanged(object sender, EventArgs e)
        {
            picFrakt.Image = CenterMandel();
        }

        private void positionZurücksetzenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            mMandelClass.ViewRect = new RectangleD(4, 3, -2.5, -1.5);
            RefreshBoxes();
            SetMandel();
        }

        private void RefreshBoxes()
        {
            txtFormPower.Text = mMandelClass.Power.ToString();
            txtFormStartIm.Text = mMandelClass.StartPos.ImaginaerPart.ToString();
            txtFormStartRe.Text = mMandelClass.StartPos.RealPart.ToString();
            txtViewHeight.Text = mMandelClass.ViewRect.Height.ToString();
            txtViewWidth.Text = mMandelClass.ViewRect.Width.ToString();
            txtViewX.Text = mMandelClass.ViewRect.X.ToString();
            txtViewY.Text = mMandelClass.ViewRect.Y.ToString();
            txtViewX2.Text = (mMandelClass.ViewRect.X + mMandelClass.ViewRect.Width).ToString();
            txtViewY2.Text = (mMandelClass.ViewRect.Y + mMandelClass.ViewRect.Height).ToString();
            cboMaxIteration.Text = mMandelClass.Iterations.ToString();
            txtImageHeight.Text = mMandelClass.ImageSize.Height.ToString();
            txtImageWidth.Text = mMandelClass.ImageSize.Width.ToString();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            RefreshBoxes();
        }

        private Rectangle GetRectFromPoints(Point pt0,Point pt1)
        {
            int minX = Math.Min(pt0.X, pt1.X);
            int minY = Math.Min(pt0.Y, pt1.Y);
            int maxX = Math.Max(pt0.X, pt1.X);
            int maxY = Math.Max(pt0.Y, pt1.Y);

            return new Rectangle(minX, minY, maxX - minX, maxY - minY);
        }
        private RectangleD GetRectFromPoints(PointD pt0,PointD pt1)
        {
            double minX = Math.Min(pt0.X, pt1.X);
            double minY = Math.Min(pt0.Y, pt1.Y);
            double maxX = Math.Max(pt0.X, pt1.X);
            double maxY = Math.Max(pt0.Y, pt1.Y);

            return new RectangleD(maxX - minX, maxY - minY, minX, minY);
        }

        private void picFrakt_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int x = Math.Max(Math.Min(e.Location.X - mMandelPos.X, mMandel.Width), 0);
                int y = Math.Max(Math.Min(e.Location.Y - mMandelPos.Y, mMandel.Height), 0);

                picFrakt.Image = CenterMandel(GetRectFromPoints( mClickPoint, new Point(x, y))); 
            }
        }

        private void txtFormPower_TextChanged(object sender, EventArgs e)
        {
            if (txtFormPower.Text != "")
            {
                int tmp;
                if (int.TryParse(txtFormPower.Text, out tmp))
                    mMandelClass.Power = tmp;
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtFormStartRe_TextChanged(object sender, EventArgs e)
        {
            if (txtFormStartRe.Text != "")
            {
                double tmp;
                if (double.TryParse(txtFormStartRe.Text, out tmp))
                    mMandelClass.StartPos.RealPart = tmp;
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtFormStartIm_TextChanged(object sender, EventArgs e)
        {
            if (txtFormStartIm.Text != "")
            {
                double tmp;
                if (double.TryParse(txtFormStartIm.Text, out tmp))
                    mMandelClass.StartPos.ImaginaerPart = tmp;
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void cboMaxIteration_TextChanged(object sender, EventArgs e)
        {
            if (cboMaxIteration.Text != "")
            {
                int tmp;
                if (int.TryParse(cboMaxIteration.Text, out tmp))
                    mMandelClass.Iterations = tmp;
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtImageWidth_TextChanged(object sender, EventArgs e)
        {
                       if (txtImageWidth.Text != "")
            {
                int tmp;
                if (int.TryParse(txtImageWidth.Text, out tmp))
                    mMandelClass.ImageSize = new Size(tmp, mMandelClass.ImageSize.Height);
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } 
        }

        private void txtImageHeight_TextChanged(object sender, EventArgs e)
        {
            if (txtImageHeight.Text != "")
            {
                int tmp;
                if (int.TryParse(txtImageHeight.Text, out tmp))
                    mMandelClass.ImageSize = new Size( mMandelClass.ImageSize.Width,tmp);
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtViewX_TextChanged(object sender, EventArgs e)
        {
            if (txtViewX.Text != "")
            {
                double tmp;
                if (double.TryParse(txtViewX.Text, out tmp))
                {
                    mMandelClass.ViewRect = new RectangleD(mMandelClass.ViewRect.Width, mMandelClass.ViewRect.Height, tmp, mMandelClass.ViewRect.Y);
                    RefreshBoxes();
                }
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtViewY_TextChanged(object sender, EventArgs e)
        {
            if (txtViewY.Text != "")
            {
                double tmp;
                if (double.TryParse(txtViewY.Text, out tmp))
                {
                    mMandelClass.ViewRect = new RectangleD(mMandelClass.ViewRect.Width, mMandelClass.ViewRect.Height, mMandelClass.ViewRect.X, tmp);
                    RefreshBoxes();
                }
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtViewWidth_TextChanged(object sender, EventArgs e)
        {
            if (txtViewWidth.Text != "")
            {
                double tmp;
                if (double.TryParse(txtViewWidth.Text, out tmp))
                {
                    mMandelClass.ViewRect = new RectangleD(tmp, mMandelClass.ViewRect.Height, mMandelClass.ViewRect.X, mMandelClass.ViewRect.Y);
                    RefreshBoxes();
                }
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void txtViewHeight_TextChanged(object sender, EventArgs e)
        {
            if (txtViewHeight.Text != "")
            {
                double tmp;
                if (double.TryParse(txtViewHeight.Text, out tmp))
                {
                    mMandelClass.ViewRect = new RectangleD(mMandelClass.ViewRect.Width, tmp, mMandelClass.ViewRect.X, mMandelClass.ViewRect.Y);
                    RefreshBoxes();
                }
                else
                    MessageBox.Show("Ungültiger Wert!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void chkColorRotation_CheckedChanged(object sender, EventArgs e)
        {
            tmrRotation.Enabled = chkColorRotation.Checked;
        }

        private void tmrRotation_Tick(object sender, EventArgs e)
        {
            ColorRotation();
            picFrakt.Image = CenterMandel();
        }

        private void ladenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Farbpalette|*.pal";

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                mMandelClass.Palette = mMandelClass.GetPalFile(dlg.FileName);

                if (mMandel != null)
                    mMandel.Palette = mMandelClass.Palette;

                picFrakt.Image = CenterMandel();
            }
        }

        private void ausschnittsdatenSpeichernToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "Ausschnittdaten|*.viw";

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                FileStream stream = new FileStream(dlg.FileName, FileMode.Create);
                BinaryWriter writer = new BinaryWriter(stream);

                writer.Write(mMandelClass.ViewRect.Width);
                writer.Write(mMandelClass.ViewRect.Height);
                writer.Write(mMandelClass.ViewRect.X);
                writer.Write(mMandelClass.ViewRect.Y);

                writer.Close();
                stream.Close();
            }
        }

        private void ausschnittsdatenLadenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Ausschnittdaten|*.viw";

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                FileStream stream = new FileStream(dlg.FileName, FileMode.Open);
                BinaryReader reader = new BinaryReader(stream);

                RectangleD rect = new RectangleD(reader.ReadDouble(), reader.ReadDouble(), reader.ReadDouble(), reader.ReadDouble());
                mMandelClass.ViewRect = rect;
                RefreshBoxes();
                SetMandel();

                reader.Close();
                stream.Close();
            }
        }
    }
}
